Package: cubature
Type: Package
Title: Adaptive Multivariate Integration over Hypercubes
Version: 2.1.4-1
VignetteBuilder: knitr
SystemRequirements: GNU make and USE_C17
URL: https://bnaras.github.io/cubature/
BugReports: https://github.com/bnaras/cubature/issues
Authors@R: c(person("Balasubramanian", "Narasimhan", role=c("aut", "cre"),
	   	   email = "naras@stat.stanford.edu"),
	     person("Manuel", "Koller", role=c("ctb"),
	           email = "koller.manuel@gmail.com"),
	     person("Dirk", "Eddelbuettel", role=c("ctb"),
	           email = "edd@debian.org"),		   
             person(c("Steven", "G."), "Johnson", role=c("aut")),
             person("Thomas", "Hahn", role=c("aut")),
	     person("Annie", "Bouvier", role=c("aut")),
	     person("Kiên", "Kiêu", role=c("aut"), email = "Kien.Kieu@jouy.inra.fr"),
	     person("Simen", "Gaure", role=c("ctb"),
	           email = "simen.gaure@frisch.uio.no"))
Description: R wrappers around the cubature C library of Steven
    G. Johnson for adaptive multivariate integration over hypercubes
    and the Cuba C library of Thomas Hahn for deterministic and
    Monte Carlo integration. Scalar and vector interfaces for 
    cubature and Cuba routines are provided; the vector interfaces
    are highly recommended as demonstrated in the package
    vignette.
License: GPL-3
Encoding: UTF-8
LinkingTo: Rcpp
Imports: Rcpp
NeedsCompilation: yes
RoxygenNote: 7.3.3
Suggests: testthat, knitr, mvtnorm, bench, rmarkdown
Packaged: 2025-11-22 00:01:16 UTC; naras
Author: Balasubramanian Narasimhan [aut, cre],
  Manuel Koller [ctb],
  Dirk Eddelbuettel [ctb],
  Steven G. Johnson [aut],
  Thomas Hahn [aut],
  Annie Bouvier [aut],
  Kiên Kiêu [aut],
  Simen Gaure [ctb]
Maintainer: Balasubramanian Narasimhan <naras@stat.stanford.edu>
Repository: CRAN
Date/Publication: 2025-11-22 06:10:10 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 15:41:11 UTC; unix
Archs: cubature.so.dSYM
