# Generated by roxygen2: do not edit by hand

export(adaptIntegrate)
export(cubintegrate)
export(cuhre)
export(default_args)
export(divonne)
export(hcubature)
export(pcubature)
export(suave)
export(vegas)
importFrom(Rcpp,evalCpp)
useDynLib(cubature)
