#' Cubature is a package for adaptive and monte-carlo multidimensional
#' integration over hypercubes
#'
#' Cubature is a package for adaptive and monte-carlo multidimensional
#' integration over hypercubes. It is a wrapper around the pure C,
#' GPLed implementations by Steven G. Johnson (cubature) and Thomas
#' Hahn (Cuba) libraries.
#'
#' @name cubature-package
#' @useDynLib cubature
#' @aliases cubature-package cubature
#' @author C code by Steven G. Johnson and Thomas Hahn, R by Balasubramanian Narasimhan, Manuel Koller, Simen Gaure, Kiên Kiêu, and Annie Bouvier
#'
#' Maintainer: Balasubramanian Narasimhan<naras@stat.stanford.edu>
#' @keywords internal
"_PACKAGE"



