# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

doCuhre <- function(nComp, f, nDim, nVec, minEval, maxEval, absTol, relTol, stateFile, key, flag) {
    .Call('_cubature_doCuhre', PACKAGE = 'cubature', nComp, f, nDim, nVec, minEval, maxEval, absTol, relTol, stateFile, key, flag)
}

doVegas <- function(nComp, f, nDim, nVec, minEval, maxEval, absTol, relTol, nStart, nIncrease, nBatch, gridNo, stateFile, seed, flag, cuba_args) {
    .Call('_cubature_doVegas', PACKAGE = 'cubature', nComp, f, nDim, nVec, minEval, maxEval, absTol, relTol, nStart, nIncrease, nBatch, gridNo, stateFile, seed, flag, cuba_args)
}

doSuave <- function(nComp, f, nDim, nVec, minEval, maxEval, absTol, relTol, nNew, nMin, flatness, stateFile, seed, flag, cuba_args) {
    .Call('_cubature_doSuave', PACKAGE = 'cubature', nComp, f, nDim, nVec, minEval, maxEval, absTol, relTol, nNew, nMin, flatness, stateFile, seed, flag, cuba_args)
}

doDivonne <- function(nComp, f, nDim, nVec, minEval, maxEval, absTol, relTol, key1, key2, key3, maxPass, border, maxChisq, minDeviation, nGiven, ldxGiven, xGiven, nExtra, peakFinder, stateFile, seed, flag, cuba_args) {
    .Call('_cubature_doDivonne', PACKAGE = 'cubature', nComp, f, nDim, nVec, minEval, maxEval, absTol, relTol, key1, key2, key3, maxPass, border, maxChisq, minDeviation, nGiven, ldxGiven, xGiven, nExtra, peakFinder, stateFile, seed, flag, cuba_args)
}

doHCubature <- function(fDim, f, xLL, xUL, maxEval, absErr, tol, vectorInterface, norm) {
    .Call('_cubature_doHCubature', PACKAGE = 'cubature', fDim, f, xLL, xUL, maxEval, absErr, tol, vectorInterface, norm)
}

doPCubature <- function(fDim, f, xLL, xUL, maxEval, absErr, tol, vectorInterface, norm) {
    .Call('_cubature_doPCubature', PACKAGE = 'cubature', fDim, f, xLL, xUL, maxEval, absErr, tol, vectorInterface, norm)
}

